//
//  SensorCommon.m
//  ANTSensorDemo
//
//  Created by Umeno Katsuhiko on 13/06/10.
//  Copyright (c) 2013年 Umeno Katsuhiko. All rights reserved.
//

#import "SensorCommon.h"

#define BC_ANT_COMMAND_TIME_OUT        2

@interface SensorCommon ()
{
    BOOL _setupDone;
    BOOL _channelClose;
    UInt16 _settingDeviceNumber;
}
@end

@implementation SensorCommon

@synthesize channel = _channel;
@synthesize deviceNumber = _deviceNumber;
@synthesize deviceType = _deviceType;
@synthesize transType = _transType;
@synthesize period = _period;
@synthesize frequency = _frequency;
@synthesize state = _state;
@synthesize delegate = _delegate;

-(id) initWithChannelNumber:(UInt8) channel;
{
    self = [super init];
    if (self) {
        _delegate = nil;
        _channel = channel;
        _deviceNumber = 0;
        _state = BC_SENSOR_STATE_INIT;
        _setupDone = NO;
        [[BCANT shardANT] registerChannelDelegate:_channel delegate:self];
    }
    
    return self;
}

-(void)dealloc
{
    NSLog(@"Sensor: dealloc type = %u", self.type);
    
    _delegate = nil;
    
    BCANT* ant = [BCANT shardANT];
    UInt8 channel = _channel;
    
    if (self.state != BC_SENSOR_STATE_INIT && self.state != BC_SENSOR_STATE_CHANNEL_CLOSED) {
        BCANTAccessory* acc = [BCANTAccessory sharedANTAccessory];
        if ([acc isAccessoryReady]) {
            dispatch_queue_t global_queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
            
            dispatch_async(global_queue, ^{
                [ant closeChannel:channel waitSecs:BC_ANT_COMMAND_TIME_OUT];
                [ant unAssignChannel:channel waitSecs:BC_ANT_COMMAND_TIME_OUT];
            });
        }
    } else {
        dispatch_queue_t global_queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
        
        dispatch_async(global_queue, ^{
            [ant unAssignChannel:channel waitSecs:BC_ANT_COMMAND_TIME_OUT];
        });
    }
    
    [[BCANT shardANT] unregisterChannelDelegate:_channel delegate:self];
}

//
// Internal Function
//
- (BOOL) startPrivate
{
    NSLog(@"Sensor: Start type=%d deviceNumber=%u", _type, _settingDeviceNumber);
    NSInteger status;
    BCANT* ant = [BCANT shardANT];
    
    if (_channelClose) {
        status = [ant closeChannel:_channel waitSecs:BC_ANT_COMMAND_TIME_OUT];
        if (status != BC_OK) {
            NSLog(@"Sensor: closeChannel failed status:%08X channel=%u", status, _channel);
            return NO;
        }
        [NSThread sleepForTimeInterval:0.5];
        
        status = [ant unAssignChannel:_channel waitSecs:BC_ANT_COMMAND_TIME_OUT];
        if (status != BC_OK) {
            NSLog(@"Sensor: unAssignChannel failed status:%08X channel=%u", status, _channel);
            return NO;
        }
    }
    
    if (_setupDone) {
        goto OPEN;
    }
    
    // Assign Channel
    status = [ant assignChannel:_channel
                    channelType:0   // Receive only
                  networkNumber:0
                       waitSecs:BC_ANT_COMMAND_TIME_OUT];
    if (status != BC_OK) {
        NSLog(@"Sensor: assignChannel failed status:%08X channel=%u", status, _channel);
        return NO;
    }
    // Set Channel ID
    status = [ant setChannelId:_channel
                     deviceNum:_settingDeviceNumber
                    deviceType:_deviceType
              transmissionType:_transType
                      waitSecs:BC_ANT_COMMAND_TIME_OUT];
    if (status != BC_OK) {
        NSLog(@"Sensor: setChannelId failed status:%08X channel=%u", status, _channel);
        return NO;
    }
    // Set RF Frequency
    status = [ant setChannelRFFreq:_channel
                            RFFreq:_frequency
                          waitSecs:BC_ANT_COMMAND_TIME_OUT];
    if (status != BC_OK) {
        NSLog(@"Sensor: setChannelRFFreq failed status:%08X channel=%u", status, _channel);
        
        return NO;
    }
    // Set Channel Period
    status = [ant setChannelPeriod:_channel
                     messagePeriod:_period
                          waitSecs:BC_ANT_COMMAND_TIME_OUT];
    if (status != BC_OK) {
        NSLog(@"Sensor: setChannelPeriod failed status:%08X channel=%u", status, _channel);
        return NO;
    }
#if 0
    // Search Timeout
    status = [ant setChannelSerachTimeout:_channel
                            searchTimeout:_searchTimeout
                                 waitSecs:BC_ANT_COMMAND_TIME_OUT];
    if (status != BC_OK) {
        NSLog(@"Sensor: setChannelSerachTimeout failed status:%08X  channel=%u", status, _channel);
        return NO;
    }
    // Low Priority Search Timeout (5sec)
    status = [ant setLowPriorityChannelSerachTimeout:_channel
                                       serachTimeout:0x02
                                            waitSecs:BC_ANT_COMMAND_TIME_OUT];
    if (status != BC_OK) {
        NSLog(@"Sensor: setLowPriorityChannelSerachTimeout failed status:%08X channel=%u", status, _channel);
        return NO;
    }
#endif
    // Setup Completed
    _setupDone = YES;
    
    
OPEN:
    // Open channel
    status = [ant openChannel:_channel
                     waitSecs:BC_ANT_COMMAND_TIME_OUT];
    if (status != BC_OK) {
        NSLog(@"Sensor: openChannel failed status:%08X  channel=%u", status, _channel);
        return NO;
    }
    
    _channelClose = NO;
    return YES;
}

-(void) start:(UInt16)deviceNumber
{
    NSLog(@"Sensor: start deviceNumber=%u", deviceNumber);
    
    _channelClose = NO;
    if (_setupDone) {
        if (_settingDeviceNumber != _deviceNumber &&
            self.state != BC_SENSOR_STATE_INIT &&
            self.state != BC_SENSOR_STATE_CHANNEL_CLOSED) {
            _channelClose = YES;
            _setupDone = NO;
        }
    }

    _state = BC_SENSOR_STATE_CHANNEL_OPENING;
    _settingDeviceNumber = deviceNumber;
    
    dispatch_queue_t global_queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
    
    // must call another thread
    dispatch_async(global_queue, ^{
        // Setup and Open Channel
        BOOL result = [self startPrivate];
        
        dispatch_async(dispatch_get_main_queue(), ^{
            if (result) {
                [self didStartCompleted];
            } else {
                [self didStartFaild];
            }
        });
    });
}

- (void) stop
{
    BCANT* ant = [BCANT shardANT];
    
    dispatch_queue_t global_queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
    
    // must call another thread
    dispatch_async(global_queue, ^{
        
        BOOL result = [ant closeChannel:_channel waitSecs:BC_ANT_COMMAND_TIME_OUT];
        
        dispatch_async(dispatch_get_main_queue(), ^{
            if (result == BC_OK) {
                [self didStopCompleted];
            } else {
                [self didStopFaild];
            }
        });
    });
}

//
// Handler function
//
- (void) didStartCompleted
{
    _state = BC_SENSOR_STATE_SEARCH;
    [self.delegate didSensorStateChanged:self.type state:BC_SENSOR_STATE_SEARCH sensor:self];
}

- (void) didStartFaild
{
    _state = BC_SENSOR_STATE_OPEN_FAILED;
    [self.delegate didSensorStateChanged:self.type state:BC_SENSOR_STATE_OPEN_FAILED sensor:self];
}

- (void) didStopCompleted
{
    _state = BC_SENSOR_STATE_INIT;
    _deviceNumber = 0;
    _transType = 0;
    
    [self.delegate didSensorStateChanged:_type state:BC_SENSOR_STATE_INIT sensor:self];
}

- (void) didStopFaild
{
    _state= BC_SENSOR_STATE_INIT;
    _deviceNumber = 0;
    _transType = 0;
    
    [self.delegate didSensorStateChanged:_type state:BC_SENSOR_STATE_INIT sensor:self];
}

- (void) didSearchTimeout
{
    _state = BC_SENSOR_STATE_SEARCH_TIMEOUT;
    [self.delegate didSensorStateChanged:_type state:BC_SENSOR_STATE_SEARCH_TIMEOUT sensor:self];
}

- (void) didGoToSearch
{
    _state = BC_SENSOR_STATE_SEARCH;
    _deviceNumber = 0;
    _transType = 0;
    
    [self.delegate didSensorStateChanged:_type state:BC_SENSOR_STATE_SEARCH sensor:self];
}

- (void) didChannelClosed
{
    _state = BC_SENSOR_STATE_CHANNEL_CLOSED;
    _deviceNumber = 0;
    
    [self.delegate didSensorStateChanged:_type state:BC_SENSOR_STATE_CHANNEL_CLOSED sensor:self];
}

- (void) didBroadcastDataReceive:(UInt8*)data length:(UInt8)length
{
    // do nothing
}

- (void) didReceiveFaile
{
    // do nothing
}

//
// BCANTDelegate function
//

- (void) channelEventFunction:(UInt8)channel eventID:(UInt8)eventID data:(UInt8*)data length:(UInt8)length
{
    NSLog(@"Sensor channelEventFunction: channel %u eventID:%02X", channel, eventID);
    
    BCANT* ant = [BCANT shardANT];
    
    switch (eventID)
    {
        case EVENT_RX_BROADCAST:
        case EVENT_RX_ACKNOWLEDGED:
        case EVENT_RX_BURST_PACKET:
            if (_channelClose) {
                break;
            }
            if (_deviceNumber != 0x0000) {
                if (_state != BC_SENSOR_STATE_CONNECTED) {
                    _state = BC_SENSOR_STATE_CONNECTED;
                    [self.delegate didSensorStateChanged:self.type state:BC_SENSOR_STATE_CONNECTED sensor:self];
                }
                //[_efficiency incrementReceive];
                [self didBroadcastDataReceive:data length:length];
            } else {
                // Get Device Number
                dispatch_queue_t global_queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
                dispatch_async(global_queue, ^{
                    UInt16  deviceNumber;
                    UInt8   deviceType;
                    UInt8   transmissionType;
                    NSInteger status = [ant getChannelID:_channel deviveNumber:&deviceNumber deviceType:&deviceType transmissionType:&transmissionType waitSecs:BC_ANT_COMMAND_TIME_OUT];
                    if (status == BC_OK) {
                        _deviceNumber = deviceNumber;
                        _transType = transmissionType;
                    }
                    
                    dispatch_async(dispatch_get_main_queue(), ^{
                        if (status == BC_OK) {
                            //[self didBroadcastDataReceive:data length:length];
                        }
                    });
                });
            }
            break;
            
        case EVENT_RX_SEARCH_TIMEOUT:
            NSLog(@"EVENT_RX_SEARCH_TIMEOUT");
            [self didSearchTimeout];
            break;
        case EVENT_RX_FAIL_GO_TO_SEARCH:
            NSLog(@"EVENT_RX_FAIL_GO_TO_SEARCH");
            [self didGoToSearch];
            break;
        case EVENT_CHANNEL_CLOSED:
            NSLog(@"EVENT_CHANNEL_CLOSED");
            [self didChannelClosed];
            break;
        case EVENT_RX_FAIL:
            //DLOG(@"EVENT_RX_FAIL");
            [self didReceiveFaile];
            break;
    }
}

- (void) notificationFunction:(UInt8)msgID data:(UInt8*)data length:(UInt8)length
{
    // do nothing
}

@end
